==> web-absence-2.0.tar.gz: absence-v2.0/cgi-bin/AbsenceAuthorization.pm <==
#---------------------------------------------------------------------
# authentication & authorization
# $Id: AbsenceAuthorization.pm 91 2009-07-14 15:58:06Z urban $
#---------------------------------------------------------------------

package AbsenceAuthorization;

use Data::Dumper;

use AbsenceDB;

BEGIN {
	# an instance has been specified
	if (exists($ENV{INSTANCE_NAME})) {
        $cookie_name = "absence-auth-$ENV{INSTANCE_NAME}";
    } else {
        $cookie_name = 'absence-auth';
    }
}

use AbsenceConfig;
use AbsenceLog;

my $AUTH		= AbsenceConfig::fetch('authentication');
#my $AUTH_TYPE	= AbsenceConfig::fetch('auth_type');
#my $SECRET		= AbsenceConfig::fetch('secret');
#my $TOP_PAGE	= AbsenceConfig::fetch('top_page');
my $SUPER_USER	= 0;

my $DEBUG		= 0;
my $VERSION		= '1.0';

sub checkSuperuser
{
	my $user_id = shift;

	if (!$AUTH || AbsenceDB::isSuperuser($user_id)) {
		return $SUPER_USER = 1;
	}

	return $SUPER_USER = 0;
}

sub dbg
{
	my $msg = shift;

	$DEBUG && abslog("DEBUG: $msg");
}

sub allowWriteForPerson
{
	my ($user_id, $pid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToObject($user_id, $pid, 'write');
}

sub allowReadForGroup
{
	my ($user_id, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'read');
}

sub allowWriteForGroup
{
	my ($user_id, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'write');
}

sub allowAdminForGroup
{
	my ($user, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'admin');
}

#-----------------------------------------------------------------
# what groups is $user allowed to see?
#-----------------------------------------------------------------
sub getReadGroups
{
	my $user_id = shift;

	$AUTH || return (AbsenceDB::getGroups());

	return AbsenceDB::findReadableGroups($user_id);
}

#-----------------------------------------------------------------
# what people is $user allowed to manage?
#-----------------------------------------------------------------
sub getAdminPeople
{
	my ($user_id, $op) = @_;

	$AUTH || return (AbsenceDB::getPeople(0));
	
	return AbsenceDB::findAdministratableObjects($user_id, $op);
}

#-----------------------------------------------------------------
# getAdminUsers() returns a list of users whom $user is allowed
# to administer.  The list depends on what the admin operation
# is.  If the operation ($op) is "delete", a list consists of
# all the users that only have access to groups for which $user
# has admin privs.  Thus, a group administrator can only delete
# a user if the user only has read/write/admin privs for groups
# that the group administrator has admin privs for...
#-----------------------------------------------------------------
sub getAdminUsers
{
	my ($user_id, $op) = @_;

	$AUTH || return AbsenceDB::getUsers();

	return AbsenceDB::findAdministratableUsers($user_id, $op);
}

#-----------------------------------------------------------------
# return true if all set $gref1 contains all members of set $gref2
#-----------------------------------------------------------------
sub superset
{
	my ($gref1, $gref2) = @_;

	my $count = 0;
	foreach my $mem (@{$gref2}) {
		if (inListN($mem, $gref1)) {
			$count++;
		}
	}

	dbg("superset: count=[$count], len(gref2) = [".@{$gref2}."]");
	($count == @{$gref2}) && return 1;
	return 0;
}

#-----------------------------------------------------------------
# adminForAllGroups() is passed a person-id to check and a list
# of groups for which the user has admin-privs.  It checks if
# the person in question is only a member of groups for which
# the user has admin-privs.  returns true if this is the case,
# false if not.
#
# ONLY USED BY getAdminPeople_old()
#-----------------------------------------------------------------
sub adminForAllGroups
{
	my ($pid, $gref) = @_;

	# first get list of groups to which person belongs
	my $pgref = AbsenceDB::getPerson($pid, 'group');
	my $count = 0;
	foreach my $g (@{$pgref}) {
		if (inListN($g, $gref)) {
			$count++;
		}
	}
	if ($count == @{$pgref}) {
		return 1;
	}
	return 0;
}

sub getAdminGroups
{
	my $user_id = shift;

	$AUTH || return AbsenceDB::getGroups();

	return AbsenceDB::findAdminGroups($user_id);
}

#-----------------------------------------------------------------
# divideGroups()
# 
# given a user-id for an administrator ($user_id) and a user-id
# for a managed object ($uid), divideGroups() determines which
# groups are administratable by $user_id (and therefore 'visible')
# and which groups are not administratable, and therefore invisible.
#-----------------------------------------------------------------
sub divideGroups
{
	my ($a_uid, $o_uid) = @_;

	my @o_groups = AbsenceDB::getUserGroups($o_uid);

	my (@vis, @invis);

	foreach my $gid (@o_groups) {
		AbsenceDB::authorizeAccessToGroup($a_uid, $gid, 'admin')
			? push(@vis, $gid)
			: push(@invis, $gid);
	}

	(\@vis, \@invis);
}

sub divideGroups2
{
	my ($user_id, $uid) = @_;

	my $oid = AbsenceDB::getOidForUid($uid)
		|| die "oid not found for uid [$uid]";

	my @o_groups = AbsenceDB::getObjectGroups($oid);
	my @admgrps = AbsenceDB::findAdminGroups($user_id);

	if (AbsenceDB::isSuperuser($user_id)) {
		return (\@o_groups, []);
	}

	my (@vis, @invis);

	foreach my $gid (@o_groups) {
		if (inListN($gid, \@admgrps)) {
			push(@vis, $gid);
		} else {
			push(@invis, $gid);
		}
	}

	(\@vis, \@invis);
}

sub adminAll
{
	my $user_id = shift;

	AbsenceDB::isSuperuser($user_id) && return 1;
	return 0;
}

sub getAdminAcls
{
	my ($a_uid, $o_uid) = @_;

	my $acls = AbsenceDB::getUserAcls($o_uid);
	$DEBUG && abslog("dump of \$acls for uid=[$o_uid]:\n".Dumper($acls)."\n--end--");
	defined($acls) || return { object => [], group => [] };

	$SUPER_USER && return $acls;

	if (AbsenceDB::isSuperuser($o_uid)) {
		# group-admin is trying to modify super-user
		return { object => [], group => [] };
	}

	my ($gvisref, $ginvref) = divideGroupAcls($a_uid, $o_uid, $acls->{group});
	my ($pvisref, $pinvref) = dividePersonAcls($a_uid, $acls->{object});

	return {object => $pvisref, group => $gvisref};
}

sub convertToInternal
{
	my $acl_ref = shift;

	$DEBUG && abslog("convertToInternal: \$acl_ref=\n".Dumper($acl_ref)."\n--end--");
	my (@g, @o);

	foreach my $oacl (@{ $acl_ref->{object} }) {
		my $targ = defined($oacl->{ref_id}) ? $oacl->{ref_id} : $oacl->{magic};
		push(@o, [ $oacl->{level} => $targ ]);
	}
	foreach my $gacl (@{ $acl_ref->{group} }) {
		my $targ = defined($oacl->{ref_id}) ? $oacl->{ref_id} : $oacl->{magic};
		push(@g, [ $oacl->{level} => $targ ]);
	}

	$DEBUG && abslog("convertToInternal: \@g=\n".Dumper(\@g)."\n--end--");
	return { object => \@o, group => \@g };
}

sub dividePersonAcls
{
	my ($a_uid, $acls) = @_;

	my @admppl = getAdminPeople($a_uid);
	my (@vis, @invis);

	foreach my $acl (@{$acls}) {
		if ($acl->{target} eq 'self') {
			push(@vis, $acl );
		} elsif (inListN($acl->{target}, \@admppl)) {
			push(@vis, $acl);
		} else {
			push(@invis, $acl);
		}
	}

	return (\@vis, \@invis);
}

sub dividePersonAcls_noconvert
{
	my ($a_uid, $o_acls) = @_;

	my @admppl = getAdminPeople($a_uid);
	my (@vis, @invis);

	foreach my $acl (@{$o_acls}) {
		if ($acl->{magic} eq 'self') {
			push(@vis, $acl);
		} elsif (inListN($acl->{ref_id}, \@admppl)) {
			push(@vis, $acl);
		} else {
			push(@invis, $acl);
		}
	}

	return (\@vis, \@invis);
}

#-------------------------------------------------------------
# selfAllowed()
#
# determine whether the magic value 'self' should be allowed
# in a group access record.
#
# This is done by first checking whether the object to be modified
# (really the associated user) already has a 'self' magic-value.  If
# not, only a super-user may add one, after that the groups to which the
# object to # be modified is a member are compared with the groups over
# which the admin-user
# has admin-privs.  The admin-user groups must be a supserset of
# the groups to which the object to be modified belongs
#-------------------------------------------------------------
sub selfAllowed
{
	my ($a_uid, $o_uid) = @_;

	defined($o_uid) || return 1;
	AbsenceDB::isSuperuser($a_uid) && return 1;
 
	AbsenceDB::userHasMagicAccess($o_uid, 'group', 'self') || return 0;

	my @o_groups = AbsenceDB::getUserGroups($a_uid);
	my @a_groups = AbsenceDB::findAdminGroups($a_uid);
	superset(\@a_groups, \@o_groups) && return 1;
	return 0;
}

sub deleteElement
{
	my($el, $lref) = @_;

	my $count = 0;
	my $match = 0;
	foreach my $tmp (@$lref) {
		if ($tmp eq $el) {
			$match = 1;
			last;
		}
		$count++;
	}

	if ($match) {
		splice(@$lref, $count, 1);
	}
}

#------------------------------------------------------------------
# divideGroupAcls() is big fun.  It sorts all the ACLs for $uid
# into two groups.  The first group contains the ACLs that $user
# is allowed to administer.  The second groups contains the ACLs
# that $user is not allowed to administer.
#------------------------------------------------------------------
sub divideGroupAcls
{
	my ($a_uid, $o_uid, $acls) = @_;

	my @admgrps = getAdminGroups($a_uid);

	my %rules;

	foreach my $acl (@{ $acls }) {
		#if ($acl->{target} =~ /^\d+$/) {
			$rules{ $acl->{target} } = $acl->{level};
		#}
	}

	#---------------------------------------------------------------
	# if $uid has an group-ACL containing "self", I must check if
	# $user has admin privs for all groups that $uid is a member of.
	# If $user does not have admin privs for all these groups,
	# the rule containing "self" must be converted into individual
	# rules containing references to real groups.
	#---------------------------------------------------------------
	my $lev;
	#if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'self')) {
	if (exists($rules{self})) {
		my @o_groups = AbsenceDB::getUserGroups($o_uid);
		if (!superset(\@admgrps, \@o_groups)) {
			foreach my $gid (@o_groups) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = $lev;
				}
			}
			delete($rules{self});
		}
	}

	#---------------------------------------------------------------
	# if there is a rule containing "all", I must do the same thing
	# as described above for "self".
	#---------------------------------------------------------------
	#if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'all')) {
	if (exists($rules{all})) {
		if ($lev != 4) {
			foreach my $gid (AbsenceDB::getGroups()) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = $lev;
				}
			}
			delete($rules{all});
		}
	}

	#---------------------------------------------------------------
	# now I can go through my list of rules and divide into the
	# two groups.  If a rule refers to a group for which $user has
	# admin privs, it goes on the visable list, otherwise on the
	# invisible list.
	#---------------------------------------------------------------
	my (@vis, @invis);
	foreach my $gid (keys(%rules)) {
		my $ref = { level => $rules{$gid}, target => $gid };
		if ($gid eq 'self') {
			push(@vis, $ref);
		} elsif ($gid eq 'all') {
			push(@invis, $ref);
		} elsif (inListN($gid, \@admgrps)) {
			push(@vis, $ref);
		} else {
			push(@invis, $ref);
		}
	}

	return (\@vis, \@invis);
}

sub divideGroupAcls_noconvert
{
	my ($a_uid, $o_uid) = @_;

	my $acls = AbsenceDB::getUserAcls($o_uid);
	my @admgrps = getAdminGroups($user);

	my %rules;

	foreach my $acl (@{ $acls->{group} }) {
		if (!defined($acl->{magic})) {
			$rules{ $acl->{ref_id} } = $acl;
		}
	}

	#---------------------------------------------------------------
	# if $o_uid has an group-ACL containing "self", I must check if
	# $a_uid has admin privs for all groups that $uid is a member of.
	# If $user does not have admin privs for all these groups,
	# the rule containing "self" must be converted into individual
	# rules containing references to real groups.
	#---------------------------------------------------------------
	my $lev;
	if ($lev = AbsenceDB::userHasMagicAccess($o_uid, 'group', 'self')) {
		my @o_groups = AbsenceDB::getUserGroups($o_uid);
		if (!superset(\@admgrps, \@o_groups)) {
			foreach my $gid (@o_groups) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = {
						level	=> $lev,
						user_id	=> $o_uid,
						ref_id	=> $gid,
						type	=> 'group',
					};
				}
			}
		} else {
			$rules{self} = {
				level	=> $lev,
				user_id	=> $o_uid,
				type	=> 'group',
				magic	=> 'self',
			};
		}
	}

	#---------------------------------------------------------------
	# if there is a rule containing "all", I must do the same thing
	# as described above for "self".
	#---------------------------------------------------------------
	if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'all')) {
		if ($lev != 4) {
			foreach my $gid (AbsenceDB::getGroups()) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = {
						level	=> $lev,
						user_id	=> $o_uid,
						ref_id	=> $gid,
						type	=> 'group',
					};
				}
			}
		} else {
			$rules{all} = {
				level	=> $lev,
				user_id	=> $o_uid,
				type	=> 'group',
				magic	=> 'all',
			};
		}
	}

	#---------------------------------------------------------------
	# now I can go through my list of rules and divide into the
	# two groups.  If a rule refers to a group for which $user has
	# admin privs, it goes on the visable list, otherwise on the
	# invisible list.
	#---------------------------------------------------------------
	my (@vis, @invis);
	foreach my $gid (keys(%rules)) {
		if ($gid eq 'self') {
			push(@vis, $rules{$gid});
		} elsif ($gid eq 'all') {
			push(@invis, $rules{$gid});
		} elsif (inListN($gid, \@admgrps)) {
			push(@vis, $rules{$gid});
		} else {
			push(@invis, $rules{$gid});
		}
	}

	return (\@vis, \@invis);
}

sub inListN
{
	my ($thing, $lref) = @_;

	foreach my $elem (@{$lref}) {
		($elem == $thing) && return 1;
	}

	return 0;
}

1;

==> web-absence-2.1.tar.gz: absence-v2.1/cgi-bin/AbsenceAuthorization.pm <==
#---------------------------------------------------------------------
# authentication & authorization
# $Id: AbsenceAuthorization.pm 111 2013-10-20 17:57:24Z urban $
# copyright Robert Urban
#---------------------------------------------------------------------

#======================================================================
#    This file is part of Absence.
#
#    Absence is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Absence is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Absence.  If not, see <http://www.gnu.org/licenses/>.
#======================================================================

package AbsenceAuthorization;

use Data::Dumper;

use AbsenceDB;

BEGIN {
	# an instance has been specified
	if (exists($ENV{INSTANCE_NAME})) {
        $cookie_name = "absence-auth-$ENV{INSTANCE_NAME}";
    } else {
        $cookie_name = 'absence-auth';
    }
}

use AbsenceConfig;
use AbsenceLog;

my $AUTH		= AbsenceConfig::fetch('authentication');
#my $AUTH_TYPE	= AbsenceConfig::fetch('auth_type');
#my $SECRET		= AbsenceConfig::fetch('secret');
#my $TOP_PAGE	= AbsenceConfig::fetch('top_page');
my $SUPER_USER	= 0;

my $DEBUG		= 0;
my $VERSION		= '2.0.1';

sub checkSuperuser
{
	my $user_id = shift;

	if (!$AUTH || AbsenceDB::isSuperuser($user_id)) {
		return $SUPER_USER = 1;
	}

	return $SUPER_USER = 0;
}

sub dbg
{
	my $msg = shift;

	$DEBUG && abslog("DEBUG: $msg");
}

sub allowWriteForPerson
{
	my ($user_id, $pid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToObject($user_id, $pid, 'write');
}

sub allowReadForGroup
{
	my ($user_id, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'read');
}

sub allowWriteForGroup
{
	my ($user_id, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'write');
}

sub allowAdminForGroup
{
	my ($user, $gid) = @_;

	$AUTH || return 1;

	return AbsenceDB::authorizeAccessToGroup($user_id, $gid, 'admin');
}

#-----------------------------------------------------------------
# what groups is $user allowed to see?
#-----------------------------------------------------------------
sub getReadGroups
{
	my $user_id = shift;

	$AUTH || return (AbsenceDB::getGroups());

	return AbsenceDB::findReadableGroups($user_id);
}

#-----------------------------------------------------------------
# what people is $user allowed to manage?
#-----------------------------------------------------------------
sub getAdminPeople
{
	my ($user_id, $op) = @_;

	$AUTH || return (AbsenceDB::getPeople(0));
	
	return AbsenceDB::findAdministratableObjects($user_id, $op);
}

#-----------------------------------------------------------------
# getAdminUsers() returns a list of users whom $user is allowed
# to administer.  The list depends on what the admin operation
# is.  If the operation ($op) is "delete", a list consists of
# all the users that only have access to groups for which $user
# has admin privs.  Thus, a group administrator can only delete
# a user if the user only has read/write/admin privs for groups
# that the group administrator has admin privs for...
#-----------------------------------------------------------------
sub getAdminUsers
{
	my ($user_id, $op) = @_;

	$AUTH || return AbsenceDB::getUsers();

	return AbsenceDB::findAdministratableUsers($user_id, $op);
}

#-----------------------------------------------------------------
# return true if all set $gref1 contains all members of set $gref2
#-----------------------------------------------------------------
sub superset
{
	my ($gref1, $gref2) = @_;

	my $count = 0;
	foreach my $mem (@{$gref2}) {
		if (inListN($mem, $gref1)) {
			$count++;
		}
	}

	dbg("superset: count=[$count], len(gref2) = [".@{$gref2}."]");
	($count == @{$gref2}) && return 1;
	return 0;
}

#-----------------------------------------------------------------
# adminForAllGroups() is passed a person-id to check and a list
# of groups for which the user has admin-privs.  It checks if
# the person in question is only a member of groups for which
# the user has admin-privs.  returns true if this is the case,
# false if not.
#
# ONLY USED BY getAdminPeople_old()
#-----------------------------------------------------------------
sub adminForAllGroups
{
	my ($pid, $gref) = @_;

	# first get list of groups to which person belongs
	my $pgref = AbsenceDB::getPerson($pid, 'group');
	my $count = 0;
	foreach my $g (@{$pgref}) {
		if (inListN($g, $gref)) {
			$count++;
		}
	}
	if ($count == @{$pgref}) {
		return 1;
	}
	return 0;
}

sub getAdminGroups
{
	my $user_id = shift;

	$AUTH || return AbsenceDB::getGroups();

	return AbsenceDB::findAdminGroups($user_id);
}

#-----------------------------------------------------------------
# divideGroups()
# 
# given a user-id for an administrator ($user_id) and a user-id
# for a managed object ($uid), divideGroups() determines which
# groups are administratable by $user_id (and therefore 'visible')
# and which groups are not administratable, and therefore invisible.
#-----------------------------------------------------------------
sub divideGroups
{
	my ($a_uid, $o_uid) = @_;

	my @o_groups = AbsenceDB::getUserGroups($o_uid);

	my (@vis, @invis);

	foreach my $gid (@o_groups) {
		AbsenceDB::authorizeAccessToGroup($a_uid, $gid, 'admin')
			? push(@vis, $gid)
			: push(@invis, $gid);
	}

	(\@vis, \@invis);
}

sub divideGroups2
{
	my ($user_id, $uid) = @_;

	my $oid = AbsenceDB::getOidForUid($uid)
		|| die "oid not found for uid [$uid]";

	my @o_groups = AbsenceDB::getObjectGroups($oid);
	my @admgrps = AbsenceDB::findAdminGroups($user_id);

	if (AbsenceDB::isSuperuser($user_id)) {
		return (\@o_groups, []);
	}

	my (@vis, @invis);

	foreach my $gid (@o_groups) {
		if (inListN($gid, \@admgrps)) {
			push(@vis, $gid);
		} else {
			push(@invis, $gid);
		}
	}

	(\@vis, \@invis);
}

sub adminAll
{
	my $user_id = shift;

	AbsenceDB::isSuperuser($user_id) && return 1;
	return 0;
}

sub getAdminAcls
{
	my ($a_uid, $o_uid) = @_;

	my $acls = AbsenceDB::getUserAcls($o_uid);
	$DEBUG && abslog("dump of \$acls for uid=[$o_uid]:\n".Dumper($acls)."\n--end--");
	defined($acls) || return { object => [], group => [] };

	$SUPER_USER && return $acls;

	if (AbsenceDB::isSuperuser($o_uid)) {
		# group-admin is trying to modify super-user
		return { object => [], group => [] };
	}

	my ($gvisref, $ginvref) = divideGroupAcls($a_uid, $o_uid, $acls->{group});
	my ($pvisref, $pinvref) = dividePersonAcls($a_uid, $acls->{object});

	return {object => $pvisref, group => $gvisref};
}

sub convertToInternal
{
	my $acl_ref = shift;

	$DEBUG && abslog("convertToInternal: \$acl_ref=\n".Dumper($acl_ref)."\n--end--");
	my (@g, @o);

	foreach my $oacl (@{ $acl_ref->{object} }) {
		my $targ = defined($oacl->{ref_id}) ? $oacl->{ref_id} : $oacl->{magic};
		push(@o, [ $oacl->{level} => $targ ]);
	}
	foreach my $gacl (@{ $acl_ref->{group} }) {
		my $targ = defined($oacl->{ref_id}) ? $oacl->{ref_id} : $oacl->{magic};
		push(@g, [ $oacl->{level} => $targ ]);
	}

	$DEBUG && abslog("convertToInternal: \@g=\n".Dumper(\@g)."\n--end--");
	return { object => \@o, group => \@g };
}

sub dividePersonAcls
{
	my ($a_uid, $acls) = @_;

	my @admppl = getAdminPeople($a_uid);
	my (@vis, @invis);

	foreach my $acl (@{$acls}) {
		if ($acl->{target} eq 'self') {
			push(@vis, $acl );
		} elsif (inListN($acl->{target}, \@admppl)) {
			push(@vis, $acl);
		} else {
			push(@invis, $acl);
		}
	}

	return (\@vis, \@invis);
}

sub dividePersonAcls_noconvert
{
	my ($a_uid, $o_acls) = @_;

	my @admppl = getAdminPeople($a_uid);
	my (@vis, @invis);

	foreach my $acl (@{$o_acls}) {
		if ($acl->{magic} eq 'self') {
			push(@vis, $acl);
		} elsif (inListN($acl->{ref_id}, \@admppl)) {
			push(@vis, $acl);
		} else {
			push(@invis, $acl);
		}
	}

	return (\@vis, \@invis);
}

#-------------------------------------------------------------
# selfAllowed()
#
# determine whether the magic value 'self' should be allowed
# in a group access record.
#
# This is done by first checking whether the object to be modified
# (really the associated user) already has a 'self' magic-value.  If
# not, only a super-user may add one, after that the groups to which the
# object to # be modified is a member are compared with the groups over
# which the admin-user
# has admin-privs.  The admin-user groups must be a supserset of
# the groups to which the object to be modified belongs
#-------------------------------------------------------------
sub selfAllowed
{
	my ($a_uid, $o_uid) = @_;

	defined($o_uid) || return 1;
	AbsenceDB::isSuperuser($a_uid) && return 1;
 
	AbsenceDB::userHasMagicAccess($o_uid, 'group', 'self') || return 0;

	my @o_groups = AbsenceDB::getUserGroups($a_uid);
	my @a_groups = AbsenceDB::findAdminGroups($a_uid);
	superset(\@a_groups, \@o_groups) && return 1;
	return 0;
}

sub deleteElement
{
	my($el, $lref) = @_;

	my $count = 0;
	my $match = 0;
	foreach my $tmp (@$lref) {
		if ($tmp eq $el) {
			$match = 1;
			last;
		}
		$count++;
	}

	if ($match) {
		splice(@$lref, $count, 1);
	}
}

#------------------------------------------------------------------
# divideGroupAcls() is big fun.  It sorts all the ACLs for $uid
# into two groups.  The first group contains the ACLs that $user
# is allowed to administer.  The second groups contains the ACLs
# that $user is not allowed to administer.
#------------------------------------------------------------------
sub divideGroupAcls
{
	my ($a_uid, $o_uid, $acls) = @_;

	my @admgrps = getAdminGroups($a_uid);

	my %rules;

	foreach my $acl (@{ $acls }) {
		#if ($acl->{target} =~ /^\d+$/) {
			$rules{ $acl->{target} } = $acl->{level};
		#}
	}

	#---------------------------------------------------------------
	# if $uid has an group-ACL containing "self", I must check if
	# $user has admin privs for all groups that $uid is a member of.
	# If $user does not have admin privs for all these groups,
	# the rule containing "self" must be converted into individual
	# rules containing references to real groups.
	#---------------------------------------------------------------
	my $lev;
	#if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'self')) {
	if (exists($rules{self})) {
		my @o_groups = AbsenceDB::getUserGroups($o_uid);
		if (!superset(\@admgrps, \@o_groups)) {
			foreach my $gid (@o_groups) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = $lev;
				}
			}
			delete($rules{self});
		}
	}

	#---------------------------------------------------------------
	# if there is a rule containing "all", I must do the same thing
	# as described above for "self".
	#---------------------------------------------------------------
	#if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'all')) {
	if (exists($rules{all})) {
		if ($lev != 4) {
			foreach my $gid (AbsenceDB::getGroups()) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = $lev;
				}
			}
			delete($rules{all});
		}
	}

	#---------------------------------------------------------------
	# now I can go through my list of rules and divide into the
	# two groups.  If a rule refers to a group for which $user has
	# admin privs, it goes on the visable list, otherwise on the
	# invisible list.
	#---------------------------------------------------------------
	my (@vis, @invis);
	foreach my $gid (keys(%rules)) {
		my $ref = { level => $rules{$gid}, target => $gid };
		if ($gid eq 'self') {
			push(@vis, $ref);
		} elsif ($gid eq 'all') {
			push(@invis, $ref);
		} elsif (inListN($gid, \@admgrps)) {
			push(@vis, $ref);
		} else {
			push(@invis, $ref);
		}
	}

	return (\@vis, \@invis);
}

sub divideGroupAcls_noconvert
{
	my ($a_uid, $o_uid) = @_;

	my $acls = AbsenceDB::getUserAcls($o_uid);
	my @admgrps = getAdminGroups($user);

	my %rules;

	foreach my $acl (@{ $acls->{group} }) {
		if (!defined($acl->{magic})) {
			$rules{ $acl->{ref_id} } = $acl;
		}
	}

	#---------------------------------------------------------------
	# if $o_uid has an group-ACL containing "self", I must check if
	# $a_uid has admin privs for all groups that $uid is a member of.
	# If $user does not have admin privs for all these groups,
	# the rule containing "self" must be converted into individual
	# rules containing references to real groups.
	#---------------------------------------------------------------
	my $lev;
	if ($lev = AbsenceDB::userHasMagicAccess($o_uid, 'group', 'self')) {
		my @o_groups = AbsenceDB::getUserGroups($o_uid);
		if (!superset(\@admgrps, \@o_groups)) {
			foreach my $gid (@o_groups) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = {
						level	=> $lev,
						user_id	=> $o_uid,
						ref_id	=> $gid,
						type	=> 'group',
					};
				}
			}
		} else {
			$rules{self} = {
				level	=> $lev,
				user_id	=> $o_uid,
				type	=> 'group',
				magic	=> 'self',
			};
		}
	}

	#---------------------------------------------------------------
	# if there is a rule containing "all", I must do the same thing
	# as described above for "self".
	#---------------------------------------------------------------
	if ($lev = AbsenceDB::userHasMagicAccess($a_uid, 'group', 'all')) {
		if ($lev != 4) {
			foreach my $gid (AbsenceDB::getGroups()) {
				if (!exists($rules{$gid}) || ($rules{$gid} < $lev)) {
					$rules{$gid} = {
						level	=> $lev,
						user_id	=> $o_uid,
						ref_id	=> $gid,
						type	=> 'group',
					};
				}
			}
		} else {
			$rules{all} = {
				level	=> $lev,
				user_id	=> $o_uid,
				type	=> 'group',
				magic	=> 'all',
			};
		}
	}

	#---------------------------------------------------------------
	# now I can go through my list of rules and divide into the
	# two groups.  If a rule refers to a group for which $user has
	# admin privs, it goes on the visable list, otherwise on the
	# invisible list.
	#---------------------------------------------------------------
	my (@vis, @invis);
	foreach my $gid (keys(%rules)) {
		if ($gid eq 'self') {
			push(@vis, $rules{$gid});
		} elsif ($gid eq 'all') {
			push(@invis, $rules{$gid});
		} elsif (inListN($gid, \@admgrps)) {
			push(@vis, $rules{$gid});
		} else {
			push(@invis, $rules{$gid});
		}
	}

	return (\@vis, \@invis);
}

sub inListN
{
	my ($thing, $lref) = @_;

	foreach my $elem (@{$lref}) {
		($elem == $thing) && return 1;
	}

	return 0;
}

1;
